@@ -6,22 +6,24 @@
 #
 # SPDX-License-Identifier: LGPL-3.0-only
 #
+# Refactor by @ariel-anieli in 2024
 
 """Glances unitary tests suite."""
 
+import functools
 import json
 import time
 import unittest
+from datetime import datetime
 
 from glances import __version__
 from glances.events_list import GlancesEventsList
 from glances.filter import GlancesFilter, GlancesFilterList
-from glances.globals import LINUX, WINDOWS, string_value_to_float, subsample
+from glances.globals import LINUX, WINDOWS, pretty_date, string_value_to_float, subsample
 from glances.main import GlancesMain
 from glances.outputs.glances_bars import Bar
 from glances.plugins.plugin.model import GlancesPluginModel
 from glances.programs import processes_to_programs
-from glances.secure import secure_popen
 from glances.stats import GlancesStats
 from glances.thresholds import (
     GlancesThresholdCareful,
@@ -54,22 +56,31 @@ class TestGlances(unittest.TestCase):
         """The function is called *every time* before test_*."""
         print('\n' + '=' * 78)
 
-    def _common_plugin_tests(self, plugin):
-        """Common method to test a Glances plugin
-        This method is called multiple time by test 100 to 1xx"""
-
-        # Reset all the stats, history and views
+    def reset_stats_history_and_views(self, plugin):
         plugin_instance = stats.get_plugin(plugin)
         plugin_instance.reset()  # reset stats
         plugin_instance.reset_views()  # reset views
         plugin_instance.reset_stats_history()  # reset history
 
-        # Check before update
-        self.assertEqual(plugin_instance.get_raw(), plugin_instance.stats_init_value)
-        self.assertEqual(plugin_instance.is_enabled(), True)
-        self.assertEqual(plugin_instance.is_disabled(), False)
-        self.assertEqual(plugin_instance.get_views(), {})
+        return plugin_instance
+
+    def zip_with(self, method, elems, values):
+        [method(elem, value) for elem, value in zip(elems, values)]
+
+    def do_checks_before_update(self, plugin_instance):
+        elems = [
+            plugin_instance.get_raw(),
+            plugin_instance.is_enabled(),
+            plugin_instance.is_disabled(),
+            plugin_instance.get_views(),
+        ]
+
+        values = [plugin_instance.stats_init_value, True, False, {}]
+
+        self.zip_with(self.assertEqual, elems, values)
+
         self.assertIsInstance(plugin_instance.get_raw(), (dict, list))
+
         if plugin_instance.history_enable() and isinstance(plugin_instance.get_raw(), dict):
             self.assertEqual(plugin_instance.get_key(), None)
             self.assertTrue(
@@ -81,86 +92,127 @@ class TestGlances(unittest.TestCase):
         elif plugin_instance.history_enable() and isinstance(plugin_instance.get_raw(), list):
             self.assertNotEqual(plugin_instance.get_key(), None)
 
-        # Update stats (add first element)
+    def update_stats(self, plugin_instance):
         plugin_instance.update()
         plugin_instance.update_stats_history()
         plugin_instance.update_views()
 
-        # Check stats
+        return plugin_instance
+
+    def is_field_in_plugin(self, plugin_instance):
+        def is_in_description(result, f):
+            return True if f in plugin_instance.fields_description else result
+
+        def is_field_in_this_description(result, i):
+            return functools.reduce(is_in_description, i, False)
+
+        return is_field_in_this_description
+
+    def is_field_in_stats(self, plugin_instance, plugin):
+        def is_field_in_this_plugin(result, f):
+            if f not in plugin_instance.get_raw():
+                print(f"WARNING: {f} field not found in {plugin} plugin stats")
+            else:
+                result = True
+
+            return result
+
+        return is_field_in_this_plugin
+
+    def check_stats(self, plugin_instance, plugin):
         self.assertIsInstance(plugin_instance.get_raw(), (dict, list))
-        if isinstance(plugin_instance.get_raw(), dict):
-            res = False
-            for f in plugin_instance.fields_description:
-                if f not in plugin_instance.get_raw():
-                    print(f"WARNING: {f} field not found in {plugin} plugin stats")
-                else:
-                    res = True
-            self.assertTrue(res)
-        elif isinstance(plugin_instance.get_raw(), list):
-            res = False
-            for i in plugin_instance.get_raw():
-                for f in i:
-                    if f in plugin_instance.fields_description:
-                        res = True
-            self.assertTrue(res)
+        if isinstance(plugin_instance.get_raw(), dict) and plugin_instance.get_raw() != {}:
+            init = False
+            is_field_present = self.is_field_in_stats(plugin_instance, plugin)
+            field_description = plugin_instance.fields_description
+            result = functools.reduce(is_field_present, field_description, init)
+
+            self.assertTrue(result)
+
+        elif isinstance(plugin_instance.get_raw(), list) and len(plugin_instance.get_raw()) > 0:
+            init = False
+            is_field_present = self.is_field_in_plugin(plugin_instance)
+            raw_from_instance = plugin_instance.get_raw()
+            result = functools.reduce(is_field_present, raw_from_instance, init)
+
+            self.assertTrue(result)
+
+        elems = [plugin_instance.get_raw(), plugin_instance.get_stats(), json.loads(plugin_instance.get_stats())]
+
+        values = [plugin_instance.get_export(), plugin_instance.get_json(), plugin_instance.get_raw()]
+
+        self.zip_with(self.assertEqual, elems, values)
 
-        self.assertEqual(plugin_instance.get_raw(), plugin_instance.get_export())
-        self.assertEqual(plugin_instance.get_stats(), plugin_instance.get_json())
-        self.assertEqual(json.loads(plugin_instance.get_stats()), plugin_instance.get_raw())
         if len(plugin_instance.fields_description) > 0:
             # Get first item of the fields_description
             first_field = list(plugin_instance.fields_description.keys())[0]
-            self.assertIsInstance(plugin_instance.get_raw_stats_item(first_field), dict)
-            self.assertIsInstance(json.loads(plugin_instance.get_stats_item(first_field)), dict)
-            self.assertIsInstance(plugin_instance.get_item_info(first_field, 'description'), str)
-        # Filter stats
+
+            elems = [
+                plugin_instance.get_raw_stats_item(first_field),
+                json.loads(plugin_instance.get_stats_item(first_field)),
+                plugin_instance.get_item_info(first_field, 'description'),
+            ]
+
+            values = [dict, dict, str]
+
+            self.zip_with(self.assertIsInstance, elems, values)
+
+    def filter_stats(self, plugin_instance):
         current_stats = plugin_instance.get_raw()
         if isinstance(plugin_instance.get_raw(), dict):
             current_stats['foo'] = 'bar'
             current_stats = plugin_instance.filter_stats(current_stats)
             self.assertTrue('foo' not in current_stats)
-        elif isinstance(plugin_instance.get_raw(), list):
+        elif isinstance(plugin_instance.get_raw(), list) and len(plugin_instance.get_raw()) > 0:
             current_stats[0]['foo'] = 'bar'
             current_stats = plugin_instance.filter_stats(current_stats)
             self.assertTrue('foo' not in current_stats[0])
 
-        # Update stats (add second element)
-        plugin_instance.update()
-        plugin_instance.update_stats_history()
-        plugin_instance.update_views()
+    def get_first_history_field(self, plugin_instance):
+        if isinstance(plugin_instance.get_raw(), dict):
+            first_history_field = plugin_instance.get_items_history_list()[0]['name']
+        elif isinstance(plugin_instance.get_raw(), list) and len(plugin_instance.get_raw()) > 0:
+            first_history_field = '_'.join(
+                [
+                    plugin_instance.get_raw()[0][plugin_instance.get_key()],
+                    plugin_instance.get_items_history_list()[0]['name'],
+                ]
+            )
 
-        # Check history
-        if plugin_instance.history_enable():
-            if isinstance(plugin_instance.get_raw(), dict):
-                first_history_field = plugin_instance.get_items_history_list()[0]['name']
-            elif isinstance(plugin_instance.get_raw(), list):
-                first_history_field = '_'.join(
-                    [
-                        plugin_instance.get_raw()[0][plugin_instance.get_key()],
-                        plugin_instance.get_items_history_list()[0]['name'],
-                    ]
-                )
+        return first_history_field
+
+    def maybe_assert_first_for_raw(self, plugin_instance, first_history_field):
+        if len(plugin_instance.get_raw()) > 0:
             self.assertEqual(len(plugin_instance.get_raw_history(first_history_field)), 2)
             self.assertGreater(
                 plugin_instance.get_raw_history(first_history_field)[1][0],
                 plugin_instance.get_raw_history(first_history_field)[0][0],
             )
 
-            # Update stats (add third element)
-            plugin_instance.update()
-            plugin_instance.update_stats_history()
-            plugin_instance.update_views()
-
+    def maybe_assert_first_for_stats(self, plugin_instance, first_history_field):
+        if len(plugin_instance.get_raw()) > 0:
             self.assertEqual(len(plugin_instance.get_raw_history(first_history_field)), 3)
             self.assertEqual(len(plugin_instance.get_raw_history(first_history_field, 2)), 2)
             self.assertIsInstance(json.loads(plugin_instance.get_stats_history()), dict)
 
-        # Check views
+    def chk_hist_maybe_add_third_elem(self, plugin_instance):
+        if plugin_instance.history_enable():
+            first_history_field = self.get_first_history_field(plugin_instance)
+            self.maybe_assert_first_for_raw(plugin_instance, first_history_field)
+
+            # Update stats (add third element)
+            plugin_instance = self.update_stats(plugin_instance)
+
+            self.maybe_assert_first_for_stats(plugin_instance, first_history_field)
+
+        return first_history_field
+
+    def check_views(self, plugin_instance, first_history_field):
         self.assertIsInstance(plugin_instance.get_views(), dict)
         if isinstance(plugin_instance.get_raw(), dict):
             self.assertIsInstance(plugin_instance.get_views(first_history_field), dict)
             self.assertTrue('decoration' in plugin_instance.get_views(first_history_field))
-        elif isinstance(plugin_instance.get_raw(), list):
+        elif isinstance(plugin_instance.get_raw(), list) and len(plugin_instance.get_raw()) > 0:
             first_history_field = plugin_instance.get_items_history_list()[0]['name']
             first_item = plugin_instance.get_raw()[0][plugin_instance.get_key()]
             self.assertIsInstance(plugin_instance.get_views(item=first_item, key=first_history_field), dict)
@@ -168,6 +220,34 @@ class TestGlances(unittest.TestCase):
         self.assertIsInstance(json.loads(plugin_instance.get_json_views()), dict)
         self.assertEqual(json.loads(plugin_instance.get_json_views()), plugin_instance.get_views())
 
+    def _common_plugin_tests(self, plugin):
+        """Common method to test a Glances plugin
+        This method is called multiple time by test 100 to 1xx"""
+
+        # Reset all the stats, history and views
+        plugin_instance = self.reset_stats_history_and_views(plugin)
+
+        # Check before update
+        self.do_checks_before_update(plugin_instance)
+
+        # Update stats (add first element)
+        plugin_instance = self.update_stats(plugin_instance)
+
+        # Check stats
+        self.check_stats(plugin_instance, plugin)
+
+        # Filter stats
+        self.filter_stats(plugin_instance)
+
+        # Update stats (add second element)
+        plugin_instance = self.update_stats(plugin_instance)
+
+        # Check history
+        first_history_field = self.chk_hist_maybe_add_third_elem(plugin_instance)
+
+        # Check views
+        self.check_views(plugin_instance, first_history_field)
+
     def test_000_update(self):
         """Update stats (mandatory step for all the stats).
 
@@ -482,6 +562,21 @@ class TestGlances(unittest.TestCase):
         self.assertTrue(gfl.is_filtered({'name': 'snake is in the place', 'username': 'nicolargo'}))
         self.assertFalse(gfl.is_filtered({'name': 'snake is in the place', 'username': 'notme'}))
 
+    def test_021_pretty_date(self):
+        """Test pretty_date"""
+        print('INFO: [TEST_021] pretty_date')
+        self.assertEqual(pretty_date(datetime(2024, 1, 1, 12, 0), datetime(2024, 1, 1, 12, 0)), 'just now')
+        self.assertEqual(pretty_date(datetime(2024, 1, 1, 11, 59), datetime(2024, 1, 1, 12, 0)), 'a min')
+        self.assertEqual(pretty_date(datetime(2024, 1, 1, 11, 55), datetime(2024, 1, 1, 12, 0)), '5 mins')
+        self.assertEqual(pretty_date(datetime(2024, 1, 1, 11, 0), datetime(2024, 1, 1, 12, 0)), 'an hour')
+        self.assertEqual(pretty_date(datetime(2024, 1, 1, 0, 0), datetime(2024, 1, 1, 12, 0)), '12 hours')
+        self.assertEqual(pretty_date(datetime(2023, 12, 20, 0, 0), datetime(2024, 1, 1, 12, 0)), '1 week')
+        self.assertEqual(pretty_date(datetime(2023, 12, 5, 0, 0), datetime(2024, 1, 1, 12, 0)), '3 weeks')
+        self.assertEqual(pretty_date(datetime(2023, 12, 1, 0, 0), datetime(2024, 1, 1, 12, 0)), '1 month')
+        self.assertEqual(pretty_date(datetime(2023, 6, 1, 0, 0), datetime(2024, 1, 1, 12, 0)), '7 months')
+        self.assertEqual(pretty_date(datetime(2023, 1, 1, 0, 0), datetime(2024, 1, 1, 12, 0)), '1 year')
+        self.assertEqual(pretty_date(datetime(2020, 1, 1, 0, 0), datetime(2024, 1, 1, 12, 0)), '4 years')
+
     def test_094_thresholds(self):
         """Test thresholds classes"""
         print('INFO: [TEST_094] Thresholds')
@@ -592,6 +687,7 @@ class TestGlances(unittest.TestCase):
         bar.percent = 110
         self.assertEqual(bar.get(), '|||||||||||||||||||||||||||||||||||||||||||| >100%')
 
+    # Error in Github Action. Do not remove the comment.
     # def test_100_system_plugin_method(self):
     #     """Test system plugin methods"""
     #     print('INFO: [TEST_100] Test system plugin methods')
@@ -623,6 +719,7 @@ class TestGlances(unittest.TestCase):
         print('INFO: [TEST_105] Test network plugin methods')
         self._common_plugin_tests('network')
 
+    # Error in Github Action. Do not remove the comment.
     # def test_106_diskio_plugin_method(self):
     #     """Test diskio plugin methods"""
     #     print('INFO: [TEST_106] Test diskio plugin methods')
@@ -633,60 +730,104 @@ class TestGlances(unittest.TestCase):
         print('INFO: [TEST_107] Test fs plugin methods')
         self._common_plugin_tests('fs')
 
-    def test_700_secure(self):
-        """Test secure functions"""
-        print('INFO: [TEST_700] Secure functions')
+    def test_200_views_hidden(self):
+        """Test hide feature"""
+        print('INFO: [TEST_200] Test views hidden feature')
+        # Test will be done with the diskio plugin, first available interface (read_bytes fields)
+        plugin = 'diskio'
+        field = 'read_bytes_rate_per_sec'
+        plugin_instance = stats.get_plugin(plugin)
+        if len(plugin_instance.get_views()) == 0 or not test_config.get_bool_value(plugin, 'hide_zero', False):
+            # No diskIO interface, test can not be done
+            return
+        # Get first disk interface
+        key = list(plugin_instance.get_views().keys())[0]
+        # Test
+        ######
+        # Init the stats
+        plugin_instance.update()
+        raw_stats = plugin_instance.get_raw()
+        # Reset the views
+        plugin_instance.set_views({})
+        # Set field to 0 (should be hidden)
+        raw_stats[0][field] = 0
+        plugin_instance.set_stats(raw_stats)
+        self.assertEqual(plugin_instance.get_raw()[0][field], 0)
+        plugin_instance.update_views()
+        self.assertTrue(plugin_instance.get_views()[key][field]['hidden'])
+        # Set field to 0 (should be hidden)
+        raw_stats[0][field] = 0
+        plugin_instance.set_stats(raw_stats)
+        self.assertEqual(plugin_instance.get_raw()[0][field], 0)
+        plugin_instance.update_views()
+        self.assertTrue(plugin_instance.get_views()[key][field]['hidden'])
+        # Set field to 1 (should not be hidden)
+        raw_stats[0][field] = 1
+        plugin_instance.set_stats(raw_stats)
+        self.assertEqual(plugin_instance.get_raw()[0][field], 1)
+        plugin_instance.update_views()
+        self.assertFalse(plugin_instance.get_views()[key][field]['hidden'])
+        # Set field back to 0 (should not be hidden)
+        raw_stats[0][field] = 0
+        plugin_instance.set_stats(raw_stats)
+        self.assertEqual(plugin_instance.get_raw()[0][field], 0)
+        plugin_instance.update_views()
+        self.assertFalse(plugin_instance.get_views()[key][field]['hidden'])
 
-        if WINDOWS:
-            self.assertIn(secure_popen('echo TEST'), ['TEST\n', 'TEST\r\n'])
-            self.assertIn(secure_popen('echo TEST1 && echo TEST2'), ['TEST1\nTEST2\n', 'TEST1\r\nTEST2\r\n'])
-        else:
-            self.assertEqual(secure_popen('echo -n TEST'), 'TEST')
-            self.assertEqual(secure_popen('echo -n TEST1 && echo -n TEST2'), 'TEST1TEST2')
-            # Make the test failed on Github (AssertionError: '' != 'FOO\n')
-            # but not on my localLinux computer...
-            # self.assertEqual(secure_popen('echo FOO | grep FOO'), 'FOO\n')
+    # def test_700_secure(self):
+    #     """Test secure functions"""
+    #     print('INFO: [TEST_700] Secure functions')
 
-    def test_800_memory_leak(self):
-        """Memory leak check"""
-        import tracemalloc
+    #     if WINDOWS:
+    #         self.assertIn(secure_popen('echo TEST'), ['TEST\n', 'TEST\r\n'])
+    #         self.assertIn(secure_popen('echo TEST1 && echo TEST2'), ['TEST1\nTEST2\n', 'TEST1\r\nTEST2\r\n'])
+    #     else:
+    #         self.assertEqual(secure_popen('echo -n TEST'), 'TEST')
+    #         self.assertEqual(secure_popen('echo -n TEST1 && echo -n TEST2'), 'TEST1TEST2')
+    #         # Make the test failed on Github (AssertionError: '' != 'FOO\n')
+    #         # but not on my localLinux computer...
+    #         # self.assertEqual(secure_popen('echo FOO | grep FOO'), 'FOO\n')
 
-        print('INFO: [TEST_800] Memory leak check')
-        tracemalloc.start()
-        # 3 iterations just to init the stats and fill the memory
-        for _ in range(3):
-            stats.update()
+    # def test_800_memory_leak(self):
+    #     """Memory leak check"""
+    #     import tracemalloc
 
-        # Start the memory leak check
-        snapshot_begin = tracemalloc.take_snapshot()
-        for _ in range(3):
-            stats.update()
-        snapshot_end = tracemalloc.take_snapshot()
-        snapshot_diff = snapshot_end.compare_to(snapshot_begin, 'filename')
-        memory_leak = sum([s.size_diff for s in snapshot_diff])
-        print(f'INFO: Memory leak: {memory_leak} bytes')
+    #     print('INFO: [TEST_800] Memory leak check')
+    #     tracemalloc.start()
+    #     # 3 iterations just to init the stats and fill the memory
+    #     for _ in range(3):
+    #         stats.update()
 
-        # snapshot_begin = tracemalloc.take_snapshot()
-        for _ in range(30):
-            stats.update()
-        snapshot_end = tracemalloc.take_snapshot()
-        snapshot_diff = snapshot_end.compare_to(snapshot_begin, 'filename')
-        memory_leak = sum([s.size_diff for s in snapshot_diff])
-        print(f'INFO: Memory leak: {memory_leak} bytes')
+    #     # Start the memory leak check
+    #     snapshot_begin = tracemalloc.take_snapshot()
+    #     for _ in range(3):
+    #         stats.update()
+    #     snapshot_end = tracemalloc.take_snapshot()
+    #     snapshot_diff = snapshot_end.compare_to(snapshot_begin, 'filename')
+    #     memory_leak = sum([s.size_diff for s in snapshot_diff])
+    #     print(f'INFO: Memory leak: {memory_leak} bytes')
 
-        # snapshot_begin = tracemalloc.take_snapshot()
-        for _ in range(300):
-            stats.update()
-        snapshot_end = tracemalloc.take_snapshot()
-        snapshot_diff = snapshot_end.compare_to(snapshot_begin, 'filename')
-        memory_leak = sum([s.size_diff for s in snapshot_diff])
-        print(f'INFO: Memory leak: {memory_leak} bytes')
-        snapshot_top = snapshot_end.compare_to(snapshot_begin, 'traceback')
-        print("Memory consumption (top 5):")
-        for stat in snapshot_top[:5]:
-            print(stat)
-            for line in stat.traceback.format():
-                print(line)
+    #     # snapshot_begin = tracemalloc.take_snapshot()
+    #     for _ in range(30):
+    #         stats.update()
+    #     snapshot_end = tracemalloc.take_snapshot()
+    #     snapshot_diff = snapshot_end.compare_to(snapshot_begin, 'filename')
+    #     memory_leak = sum([s.size_diff for s in snapshot_diff])
+    #     print(f'INFO: Memory leak: {memory_leak} bytes')
+
+    #     # snapshot_begin = tracemalloc.take_snapshot()
+    #     for _ in range(300):
+    #         stats.update()
+    #     snapshot_end = tracemalloc.take_snapshot()
+    #     snapshot_diff = snapshot_end.compare_to(snapshot_begin, 'filename')
+    #     memory_leak = sum([s.size_diff for s in snapshot_diff])
+    #     print(f'INFO: Memory leak: {memory_leak} bytes')
+    #     snapshot_top = snapshot_end.compare_to(snapshot_begin, 'traceback')
+    #     print("Memory consumption (top 5):")
+    #     for stat in snapshot_top[:5]:
+    #         print(stat)
+    #         for line in stat.traceback.format():
+    #             print(line)
 
     def test_999_the_end(self):
         """Free all the stats"""
